<?php

try {

	$timeStart = microtime(true);

	require_once dirname(__FILE__).'/../class/class.Core.php';

	// init core app
	Core::init("api");

	// set header
	header("HTTP/1.0 200 OK");
	header("Content-Type : application/json");

	// explode URL
	$askedUrl = $_SERVER['SCRIPT_NAME'];

	if(!preg_match('/(\/.*){4,}/', $askedUrl)) {
		throw new CoreException(CoreException::UNAUTHORIZED_API_METHOD);
	}

	list($version, $module, $action) = array_slice(explode('/', $askedUrl), 2);
	$verb = strtolower($_SERVER['REQUEST_METHOD']);

	//check if this methods is authorized through public interface
	if (in_array($action, AccessRights::FORBIDDEN_METHODS)) {
		throw new CoreException(CoreException::UNAUTHORIZED_API_METHOD);
	}

	// import correct json class
	$modulePath = API_DIR.'/'.$version.'/object/json.'.$module.'.php';
	if (!file_exists($modulePath)) {
		throw new CoreException(CoreException::INVALID_API_METHOD);
	}
	require_once $modulePath;

	// check if class exists
	if (!class_exists($module)) {
		throw new CoreException(CoreException::INVALID_API_METHOD);	
	}
	// get reflection class
	$methodName = $verb.'_'.$action;
	$class = new ReflectionClass($module);

	if (!$class->hasMethod($methodName)) {
		throw new CoreException(CoreException::INVALID_API_METHOD);	
	}

	// get reflection method
	$method = new ReflectionMethod($class->getName(), $methodName);

	if (!$method->isPublic()) {
		throw new CoreException(CoreException::UNAUTHORIZED_API_METHOD);
	}

	//get reflection object
	$instance = $class->newInstanceArgs();

	// sanitize arguments
	$instance->sanitizeArgs();

	// call method
	$result = $instance->$methodName();

	//check results
	// TODO ...

	//return results
	$time = round( (microtime(true) - $timeStart) *1000, 2);
	$result['elasped_time'] = $time;
	echo json_encode($result);


} catch (CoreException $e) {

	error_log($e);
	header("HTTP/1.0 404 Not Found");
	header("Content-Type : application/json");

	$data = array(
		"status" => ApiStatus::API_FAILURE_STATUS,
		"message" => $e->getMessage()
	);

	echo json_encode($data);
}